"""
Speech-to-Text providers for Esperanto.
This module exports the base speech-to-text model class.
"""

from esperanto.providers.stt.base import SpeechToTextModel

__all__ = [
    "SpeechToTextModel",
]
